//
//  PTBitmapVC.m
//
//  Created by midmirror on 15/12/28.
//  Copyright © 2015年 midmirror. All rights reserved.
//

#import "PTBitmapVC.h"
#import "PTConnectVC.h"
#import <Masonry/Masonry.h>
#import "WirelessPrinter-Swift.h"
#import "ProgressHUD.h"
#import "PTDisconnectVC.h"
#import "PTIcon.h"
#import "PTScaleImage.h"
#import "PTPrinterPort.h"

@interface PTBitmapVC ()

@property(strong,nonatomic,readwrite) FlatPopView *popView;
@property(strong,nonatomic,readwrite) UITableView *tableView;
@property(strong,nonatomic,readwrite) UIImage *photo;
@property(strong,nonatomic,readwrite) UIImage *bitmapSelected;
@property(strong,nonatomic,readwrite) NSArray *bitmapArray;
@property(strong,nonatomic,readwrite) NSIndexPath *indexPathSelected;
@property(strong,nonatomic,readwrite) PTDisconnectVC *disconnectVC;
@property(strong,nonatomic,readwrite) UISegmentedControl *compressModeSegment;
@property(strong,nonatomic,readwrite) UISegmentedControl *imageTypeSegment;

@end

@implementation PTBitmapVC

- (id)init {
    
    if (self = [super init]) {
        self.title = NSLocalizedString(@"Bitmap", nil);
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.navigationController.navigationBar.titleTextAttributes = @{NSForegroundColorAttributeName: [UIColor darkGrayColor]};
    
    PTSymbolFont *printer = [[PTSymbolFont alloc] initWithCode:icon_printer size:30 fontName:icon_Name color:FLAT_COLOR_BLUE];
    self.navigationItem.rightBarButtonItem =
    [[UIBarButtonItem alloc] initWithImage:[printer image:CGSizeMake(30, 33)] style:UIBarButtonItemStylePlain target:self action:@selector(printAction)];
    
    _bitmapArray = @[@"logo.jpg",@"cainiao.jpg",@"pander.jpg",@"emeya.jpg"];
    [self setupFootTable];
}

- (void)viewWillAppear:(BOOL)animated {
    
    [super viewWillAppear:YES];
    [self addDisconnectVC];
    
    [[PTDispatcher share] whenUnconnect:^{
        [self addDisconnectVC];
    }];
    
    [[PTDispatcher share] whenConnectSuccess:^{
        [self removeDisconnectVC];
    }];
    
    self.navigationController.navigationBar.backgroundColor = FLAT_COLOR_GRAY;
}

- (void)popFlashConnectVC {
    
    PTFlashConnectVC *flastConnectVC = [[PTFlashConnectVC alloc] init];
    [self presentViewController:flastConnectVC animated:true completion:^{
        
    }];
}

- (void)viewWillDisappear:(BOOL)animated {
    
    [super viewWillDisappear:YES];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self removeDisconnectVC];
}

- (void)popConnectVC {
    PTConnectVC *connectVC = [[PTConnectVC alloc] init];
    [self presentViewController:connectVC animated:YES completion:^{
        
    }];
}

- (void)addDisconnectVC
{
    if ([PTDispatcher share].mode == PTDispatchModeUnconnect) {
        self.disconnectVC = [[PTDisconnectVC alloc] init];
        [self addChildViewController:self.disconnectVC];
        [self.view addSubview:self.disconnectVC.view];
        [self.disconnectVC didMoveToParentViewController:self];
    }
}

- (void)removeDisconnectVC
{
    [self.disconnectVC willMoveToParentViewController:nil];
    [self.disconnectVC.view removeFromSuperview];
    [self.disconnectVC removeFromParentViewController];
}

/** 设置下方的表视图 */
- (void)setupFootTable {
    
    _tableView = [[UITableView alloc] initWithFrame:self.view.bounds style:UITableViewStyleGrouped];
    _tableView.dataSource = self;
    _tableView.delegate = self;
    [self.view addSubview:_tableView];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/** 根据当前的指令集选择打印功能 */
- (void)printAction {
    
    switch ([PTDispatcher share].printerConnected.commandMode) {
        case PTCommandModeTSPL:   [self printBitmapTSPL];      break;
            
        default:
            break;
    }
}
/** TSC 指令集打印图片 */
- (void)printBitmapTSPL {
    
    if (_bitmapSelected) {
        
        UIImageView *resultImageView = [[UIImageView alloc] initWithFrame:CGRectMake(30, 500, 500, 375)];
        [self.view addSubview:resultImageView];
        
        PTCommandTSPL *tsc = [[PTCommandTSPL alloc] init];
        NSData *bmpData = [PTBitmap getImageData:_bitmapSelected.CGImage mode:PTBitmapModeDithering compress:PTBitmapCompressModeNone];
        NSData *bmpNagativeData = [PTBitmap negativeData:bmpData];

        [tsc setPrintAreaSizeWithWidth:100 Height:80];
        [tsc setCLS];
        [tsc addBitmapWithXPos:0 YPos:0 width:_bitmapSelected.size.width height:_bitmapSelected.size.height Mode:1 imageData:bmpNagativeData];
        [tsc printWithSets:1 Copies:1];
        
        [PTPrinterPort sendData:tsc.cmdData];
    }
}

/** 打开相册 */
- (void)openAlbum {
    UIImagePickerController *imagePickerCtr = [[UIImagePickerController alloc] init];
    
    imagePickerCtr.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    imagePickerCtr.delegate = self;
    imagePickerCtr.allowsEditing = YES;
    [self presentViewController:imagePickerCtr animated:YES completion:^{
        
    }];
}

/** 打开相机 */
- (void)openCamera {
    
    UIImagePickerController *imagePickerCtr = [[UIImagePickerController alloc] init];
    BOOL isCamera = [UIImagePickerController isCameraDeviceAvailable:UIImagePickerControllerCameraDeviceRear];
    if (!isCamera) {
//        DebugLog(@"不支持摄像头");
        return;
    }
    imagePickerCtr.sourceType = UIImagePickerControllerSourceTypeCamera;
    imagePickerCtr.delegate = self;
    imagePickerCtr.allowsEditing = YES;
    [self presentViewController:imagePickerCtr animated:YES completion:^{
        
    }];
}

#pragma mark UIImagePickerControllerDelegate
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info;
{
    //    UIImagePickerControllerEditedImage 编辑图片
    //    UIImagePickerControllerOriginalImage 原始图片
    
    _photo = [info objectForKey:UIImagePickerControllerEditedImage];
    [_tableView reloadData];
    
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}
- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker;
{
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

#pragma mark UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView;              // Default is 1 if not implemented
{
    if (_photo) {
        return 4;
    }else {
        return 3;
    }
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    switch (section) {
        case 0: return 2;                   break;
        case 1: return 2;                   break;
        case 2: return _bitmapArray.count;  break;
        case 3: return 1;   break;
        default:return 1;   break;
    }
}

- (nullable UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section
{
    UIView *footView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, SCREEN_WIDTH, 30)];
    
    UILabel *footLabel = [[UILabel alloc] init];
    if (section == 0) {
        footLabel.frame = CGRectMake(20, 0, SCREEN_WIDTH, 30);
        footLabel.text = NSLocalizedString(@"Only 'MPT-8' support TIFF compress mode", nil);
        footLabel.font = [UIFont systemFontOfSize:[FlatFit subTitleFontSize]];
    }
    footLabel.textColor = [UIColor grayColor];
    [footView addSubview:footLabel];
    
    return footView;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {

    return 30;
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section
{
    if (section == 0) {
        return 30;
    }else {
        return 1;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath;
{
    static NSString *iden = @"PTBitmapCell";
    
    UITableViewCell *tableCell;
    if (indexPath.section == 0) {
        
        tableCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"bitmapVCCellIden"];
    }else {
        
        tableCell = [tableView dequeueReusableCellWithIdentifier:iden];
        if (tableCell == nil) {
            tableCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:iden];
        }
    }
    
    //tableCell.textLabel.font = [UIFont fontWithName:font_light size:[FlatFit titleFontSize]];
    //tableCell.detailTextLabel.font = [UIFont fontWithName:font_Thin size:[FlatFit subTitleFontSize]];
    
    if (indexPath.section == 0) {
        
        NSArray *optionTitles = @[@"Compress Mode",@"Bitmap Type"];
        tableCell.textLabel.text = NSLocalizedString(optionTitles[indexPath.row], nil);
        
        NSArray *compressTitles = @[@[@"NULL",@"TIFF"],@[NSLocalizedString(@"Halftone", nil),NSLocalizedString(@"Binary", nil)]];

        UISegmentedControl *compressSegment = [[UISegmentedControl alloc] initWithItems:compressTitles[indexPath.row]];
        compressSegment.tintColor = FLAT_COLOR_BLUE;
        compressSegment.selectedSegmentIndex = 0;
        
        if (indexPath.row == 0) {
            _compressModeSegment = compressSegment;
            tableCell.accessoryView =  _compressModeSegment;
        }else if (indexPath.row ==1) {
            _imageTypeSegment = compressSegment;
            tableCell.accessoryView = _imageTypeSegment;
        }
        
    }else if (indexPath.section == 1) {
        
        NSArray *funcArray = @[NSLocalizedString(@"Open Album", nil),NSLocalizedString(@"Open Camera", nil)];
        NSArray *detailArray = @[NSLocalizedString(@"Open_Album_Detail_Title", nil),
                                 NSLocalizedString(@"Open_Camera_Detial_Title", nil)];
        tableCell.textLabel.text =funcArray[indexPath.row];
        tableCell.detailTextLabel.text = detailArray[indexPath.row];
        
    }else if (indexPath.section == 2){
        
        NSString *path = [[NSBundle mainBundle] pathForResource:_bitmapArray[indexPath.row] ofType:nil];
        UIImage *image = [UIImage imageWithContentsOfFile:path];
        tableCell.imageView.image = image;
        tableCell.imageView.contentMode = UIViewContentModeScaleAspectFit;
        tableCell.textLabel.text = _bitmapArray[indexPath.row];
        
    }else {
        
        tableCell.imageView.image = _photo;
        tableCell.textLabel.text = @"将要打印的图片";
    }
    
    tableCell.detailTextLabel.textColor = [UIColor grayColor];
    return tableCell;
}


#pragma mark UITableViewDelegate

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (indexPath.section == 2) {
        return 60;
    } else {
        return [FlatFit rowHeight];
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath;
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];//取消选中 cell
    
    if (indexPath.section == 0) {
        
    } else if (indexPath.section == 1) {
        
        switch (indexPath.row) {
            case 0: [self openAlbum];       break;
            case 1: [self openCamera];      break;
            case 2: [self printAction];     break;
                
            default:                        break;
        }
    } else {
        
        if(indexPath.section == 2) {
            _bitmapSelected = [UIImage imageNamed:_bitmapArray[indexPath.row]];
            
        }else if(indexPath.section == 3) {
            _bitmapSelected = _photo;
        }

        UITableViewCell *lastCell = [tableView cellForRowAtIndexPath:_indexPathSelected];
        lastCell.accessoryType = UITableViewCellAccessoryNone;
        
        //    新选中的一行，通过 indexPath 找到 cell
        UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
        _indexPathSelected = indexPath;
        
        [tableView reloadData];
        
        /*
        PTPrintVC *printVC = [[PTPrintVC alloc] init];
        printVC.image = _bitmapSelected;
        printVC.mediaType = PTPrinterMediaTypeImage;
        printVC.sourceData = [self printBitmapCPCL];
        printVC.hidesBottomBarWhenPushed = true;
        [self.navigationController pushViewController:printVC animated:true];
         */
    }
}

@end
