//
//  PTConnectVC.m
//  WirelessPrinter
//
//  Created by midmirror on 16/2/24.
//  Copyright © 2016年 midmirror. All rights reserved.
//

#import "PTConnectVC.h"
#import <SystemConfiguration/CaptiveNetwork.h>
#import <Masonry/Masonry.h>
#import "MJRefresh.h"
#import "ProgressHUD.h"
#import "PTPrinterPort.h"
#import "PTSetting.h"

@interface PTConnectVC ()

@property(strong,nonatomic,readwrite) UITableView *tableView;
@property(strong,nonatomic,readwrite) UIView *headView;
@property(strong,nonatomic,readwrite) NSIndexPath *indexPathSelected;
@property(assign,nonatomic,readwrite) NSInteger sectionHeaderHeight;
@property(strong,nonatomic,readwrite) UISwitch *distanceSwitcher;
@property(strong,nonatomic,readwrite) UISwitch *showHPRTSwitcher;
@property(strong,nonatomic,readwrite) UISwitch *setFeedSwitch;
@property(strong,nonatomic,readwrite) UISegmentedControl *segment;

@property(strong,nonatomic,readwrite) NSArray *blePrinters;
@property(strong,nonatomic,readwrite) NSArray *wifiPrinters;
@property(strong,nonatomic,readwrite) PTRouter *router;

/** 网卡的 MAC 对应的产商 */
@property(strong,nonatomic,readwrite) NSMutableDictionary *ouiBaseDict;

@end

@implementation PTConnectVC

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.view.backgroundColor = FLAT_COLOR_COULD;
    
    CGFloat navigationBarHeight = [FlatFit navigationBarHeight];
    
    _headView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, SCREEN_WIDTH, navigationBarHeight)];
    _headView.backgroundColor = FLAT_COLOR_GRAY;
    [self.view addSubview:_headView];
    
    _tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, navigationBarHeight, SCREEN_WIDTH, SCREEN_HEIGHT*0.9-navigationBarHeight) style:UITableViewStyleGrouped];
    _tableView.dataSource = self;
    _tableView.delegate = self;
    
    [self.view addSubview:_tableView];
    [self setupRefresh];            // 刷新蓝牙
    
    UIView *footView = [[UIView alloc] init];
    [self.view addSubview:footView];
    
    [footView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.bottom.equalTo(self.view);
        make.size.mas_equalTo(CGSizeMake(SCREEN_WIDTH, SCREEN_HEIGHT*0.1));
    }];
    
    CGFloat buttonHeight = [FlatFit getMainButtonHeight];
    
    UIButton *closeButton = [[UIButton alloc] init];
    [closeButton addTarget:self action:@selector(closeView) forControlEvents:UIControlEventTouchUpInside];
    closeButton.layer.cornerRadius = buttonHeight/2;
    closeButton.backgroundColor = FLAT_COLOR_GRAY;
    closeButton.titleLabel.font = [UIFont systemFontOfSize:[FlatFit titleFontSize]];
    [closeButton setTitle:NSLocalizedString(@"Cancel", nil) forState:UIControlStateNormal];
    [footView addSubview:closeButton];
    
    CGFloat buttonWidth = SCREEN_WIDTH/2.;
    [closeButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(footView.mas_centerX);
        make.centerY.equalTo(footView);
        make.size.mas_equalTo(CGSizeMake(buttonWidth, buttonHeight));
    }];
    
    [self initSegment];
    
    [NSTimer scheduledTimerWithTimeInterval:1.5 target:self selector:@selector(reloadBLETable) userInfo:nil repeats:true];
    
    self.showHPRTSwitcher = [[UISwitch alloc] initWithFrame:CGRectMake(0, 0, 44, 44)];
    self.showHPRTSwitcher.on = [[PTSetting share] getScanSettingWithKey:setting_show_hprt];
    
    self.distanceSwitcher = [[UISwitch alloc] initWithFrame:CGRectMake(0, 0, 44, 44)];
    self.distanceSwitcher.on = [[PTSetting share] getScanSettingWithKey:setting_show_distance];
    
    self.setFeedSwitch = [[UISwitch alloc] initWithFrame:CGRectMake(0, 0, 44, 44)];
    self.setFeedSwitch.on = [[PTSetting share] getScanSettingWithKey:setting_feed_one_line];
}

- (void)showHPRTPrinterOnly {
    
    [self scanBLEPrinter];
    [self.tableView reloadData];
}

- (void)initSegment {
    
    NSArray *segmentTitles = @[@"Setting",@"BLE",@"WiFi"];
    self.segment = [[UISegmentedControl alloc] initWithItems:segmentTitles];
    self.segment.tintColor = [UIColor whiteColor];
    self.segment.selectedSegmentIndex = 1;
    [self.segment addTarget:self action:@selector(connectSwitcher:) forControlEvents:UIControlEventValueChanged];
    [_headView addSubview:self.segment];
    
    CGSize segmentSize;
    if (SCREEN_RESOLUTION == iPad) {
        segmentSize = CGSizeMake(70 * segmentTitles.count, 30);
    }else {
        segmentSize = CGSizeMake(50 * segmentTitles.count, 25);
    }
    [self.segment mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(_headView).offset(-15);
        make.centerX.equalTo(_headView);
        make.size.mas_equalTo(segmentSize);
    }];
}

- (void)viewWillAppear:(BOOL)animated {
    
    [super viewWillAppear:YES];
}

- (void)viewDidDisappear:(BOOL)animated {
    
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)removeLayer:(CALayer *)layer
{
    [layer removeFromSuperlayer];
}

- (void)closeView {
    
    //保存一下当前设置
    [[PTSetting share] saveScanSettingWithKey:setting_show_hprt value:self.showHPRTSwitcher.on];
    [[PTSetting share] saveScanSettingWithKey:setting_show_distance value:self.distanceSwitcher.on];
    [[PTSetting share] saveScanSettingWithKey:setting_feed_one_line value:self.setFeedSwitch.on];
    [[PTSetting share] sychronize];
    
    [[PTDispatcher share] stopScanBluetooth];
    [ProgressHUD dismiss];
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (void)viewWillDisappear:(BOOL)animated {
    
   [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [_headView.layer removeAllAnimations];
    [super viewWillDisappear:YES];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView;              // Default is 1 if not implemented
{
    return 1;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return [FlatFit rowHeight];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section;
{
    if (self.segment.selectedSegmentIndex == 0) {
        
        return 3;
    } else if (self.segment.selectedSegmentIndex == 1) {
        
        return self.blePrinters.count;
        
    } else if (self.segment.selectedSegmentIndex == 2) {
        
        return self.wifiPrinters.count;
    } else {
        
        return 1;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *iden = @"connectCell";
    FlatTableCell *tableCell;
    
    tableCell = [tableView dequeueReusableCellWithIdentifier:iden];
    if (tableCell == nil) {
        tableCell = [[FlatTableCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:iden];
    }
    
    tableCell.textLabel.text = @"";
    tableCell.detailTextLabel.text = @"";
    if (self.segment.selectedSegmentIndex != 1) {
        tableCell.imageView.image = nil;
    }
    
    tableCell.accessoryView = nil;
    tableCell.rightLabel.text = nil;
    tableCell.rightLabel.textColor = [UIColor blackColor];

    if (self.segment.selectedSegmentIndex == 0) {
        tableCell.textLabel.font = [UIFont fontWithName:font_light size:[FlatFit titleFontSize]-4];
        [self updateSettingCell:tableCell indexPath:indexPath];
    } else if (self.segment.selectedSegmentIndex == 1) {
        [self updateBLECell:tableCell indexPath:indexPath];
    } else if (self.segment.selectedSegmentIndex == 2) {
       [self updateWiFiCell:tableCell indexPath:indexPath];
    }
    
    return tableCell;
}

- (void)updateSettingCell:(FlatTableCell *)tableCell indexPath:(NSIndexPath *)indexPath {
    
    if (indexPath.row == 0) {
        
        tableCell.textLabel.text = NSLocalizedString(@"Show HPRT Only",nil);
        [self.showHPRTSwitcher addTarget:self action:@selector(showHPRTPrinterOnly) forControlEvents:UIControlEventValueChanged];
        tableCell.accessoryView = self.showHPRTSwitcher;
        
    } else if (indexPath.row == 1) {
        
        tableCell.textLabel.text = NSLocalizedString(@"Show Distance",nil);
        [self.distanceSwitcher addTarget:self action:@selector(reloadBLETable) forControlEvents:UIControlEventValueChanged];
        tableCell.accessoryView = self.distanceSwitcher;
    } else if (indexPath.row == 2) {
        
        tableCell.textLabel.text = NSLocalizedString(@"Enable Printer to React when Connected",nil);
        tableCell.accessoryView = self.setFeedSwitch;
    }
}

- (void)updateWiFiCell:(FlatTableCell *)tableCell indexPath:(NSIndexPath *)indexPath {
    
    if (self.router.connected) {
        
        PTPrinter *printer = self.wifiPrinters[indexPath.row];
        NSString *company = self.ouiBaseDict[printer.MACKey];
        
        if ([printer.IP isEqualToString:self.router.localIP]) { // 如果搜索到的 IP 是本机（iOS） IP
            
            NSString *thisDevice = [[NSString alloc] initWithFormat:@"%@ (%@)",company,[[UIDevice currentDevice] name]];
            tableCell.textLabel.text = thisDevice;
        } else {
            
            tableCell.textLabel.text = company;
        }
        
        tableCell.detailTextLabel.text = printer.MAC;
        tableCell.rightLabel.text = printer.IP;
        //tableCell.detailTextLabel.textColor = [UIColor grayColor];
        
    }else {
        tableCell.textLabel.text = NSLocalizedString(@"WiFi Offline", nil);
        tableCell.textLabel.textColor = [UIColor grayColor];
        tableCell.detailTextLabel.text = NSLocalizedString(@"please connect printer WiFi in system setting", nil);
    }
}

- (void)updateBLECell:(FlatTableCell *)tableCell indexPath:(NSIndexPath *)indexPath {
    
    PTPrinter *printer;
    if (self.blePrinters.count > 0) {
        printer = self.blePrinters[indexPath.row];
    }
    
    tableCell.textLabel.text = printer.name;
    if (![printer.MAC isEqualToString:@"(null)"]) {
        tableCell.detailTextLabel.text = printer.MAC;
    }
    
    NSArray *strengthImages = @[@"signal_strength_0",@"signal_strength_1",@"signal_strength_2",@"signal_strength_3",@"signal_strength_4",@"signal_strength_5"];
    tableCell.imageView.image = [UIImage imageNamed:strengthImages[printer.strength.integerValue]];
    
    NSString *rightLabelText = [self getTimeStatusWithUUID:printer.UUID];
    UIColor *rightLabelColor = [UIColor blackColor];
    
    if (self.distanceSwitcher.on == true) { // 是否显示距离
        
        NSInteger distanceValue = printer.distance.integerValue;
        if (distanceValue < 10) {
            rightLabelColor = FLAT_COLOR_GREEN;
        }else if (distanceValue >= 10 && distanceValue < 80) {
            rightLabelColor = FLAT_COLOR_GRAY;
        }else if (distanceValue >= 80) {
            rightLabelColor = [UIColor whiteColor];
        }
        rightLabelText = [[NSString alloc] initWithFormat:@"%.2f m",printer.distance.floatValue];
    }
    
    tableCell.rightLabel.text = rightLabelText;
    tableCell.rightLabel.textColor = rightLabelColor;
}

- (void)reloadBLETable {
    
    [self.tableView reloadData];
}

#pragma mark - UITableViewDelegate

- (nullable NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    NSArray *titles = @[@[@"Setting"],@[@"Bluetooth"],@[@"WiFi"]];
    return NSLocalizedString(titles[self.segment.selectedSegmentIndex][section], nil);
}

- (nullable NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section
{
    if (self.segment.selectedSegmentIndex == 1 && section == 0) {
        return NSLocalizedString(@"Distance cal by signal strength", nil);
    } else {
        return nil;
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];//取消选中 cell
    _indexPathSelected = indexPath;
    
    if (self.segment.selectedSegmentIndex == 2) {
        
        PTPrinter *printer = self.wifiPrinters[indexPath.row];
        NSString *company = self.ouiBaseDict[printer.MACKey];
        
        UIAlertController* alert = [UIAlertController alertControllerWithTitle:@"Warning"
                                                                       message:[NSString stringWithFormat:@"Connect: %@  Port:9100",printer.IP]
                                                                preferredStyle:UIAlertControllerStyleAlert];
        
        UIAlertAction* defaultAction = [UIAlertAction actionWithTitle:@"Connect" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            
            [PTDispatcher share].printerConnected.name = [[NSString alloc] initWithFormat:@"%@ %@",printer.IP,company];
            [ProgressHUD show:printer.IP];
            
            printer.port = @"9100";
            [[PTDispatcher share] connectPrinter:printer];
            
            [[PTDispatcher share] whenConnectSuccess:^{
                [ProgressHUD showSuccess:@"连接成功"];
                [self closeView];
            }];
            
            [[PTDispatcher share] whenConnectFailure:^{
                [ProgressHUD showError:@"连接错误"];
            }];
        }];
        
        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
            
        }];
        
        [alert addAction:defaultAction];
        [alert addAction:cancelAction];
        
        [self presentViewController:alert animated:YES completion:nil];
        
    } else if (self.segment.selectedSegmentIndex == 1){
        
        [self connectBLE:indexPath];
    } else if (self.segment.selectedSegmentIndex == 0) {
        
    }
}

- (void)connectBLE:(NSIndexPath *)indexPath {
    
    PTPrinter *printer = self.blePrinters[indexPath.row];
    [ProgressHUD show:[NSString stringWithFormat:@"%@",printer.name]];
    
    [[PTDispatcher share] connectPrinter:printer];
    
    [[PTDispatcher share] whenConnectSuccess:^{
        [ProgressHUD showSuccess:@"连接成功"];
        [PTPrinterPort saveBluetoothWithUUID:printer.UUID name:printer.name];
        [self closeView];
    }];
    
    [[PTDispatcher share] whenConnectFailure:^{
        [ProgressHUD showError:@"连接错误"];
    }];
}

#pragma mark - PTDispatchProtocol

- (void)sortPrinters {
    
    NSDictionary *deviceRecordDict = [[NSUserDefaults standardUserDefaults] objectForKey:@"deviceRecord"];
    for (PTPrinter *printer in self.blePrinters) {
        
        if (deviceRecordDict[printer.UUID]) {
            
            printer.lastTime = deviceRecordDict[printer.UUID][@"time"];
        }
    }
    NSSortDescriptor *descTime = [NSSortDescriptor sortDescriptorWithKey:@"lastTime" ascending:NO];
    NSSortDescriptor *descName = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES];
    self.blePrinters  = [self.blePrinters sortedArrayUsingDescriptors: @[descTime,descName]];
}

/**
 * 根据保存在 UserDefault 中的 deviceReord，用 uuid 取出上一次连接的时间。计算间隔
 *
 *  @param uuidKey 扫描到外设的 uuid
 */
- (NSString *)getTimeStatusWithUUID:(NSString * __nonnull)uuidKey
{
    NSDictionary *deviceRecordDict = [[NSUserDefaults standardUserDefaults] objectForKey:@"deviceRecord"];
    
    if (deviceRecordDict[uuidKey]) {
        
        double timeInterval = [deviceRecordDict[uuidKey][@"time"] doubleValue];
        double currentTime = [[PTPrinterPort getCurrentTimeIntervalSince1970] doubleValue];
        double interval =  currentTime - timeInterval;
        
        NSString *timeStamp;
        if (interval <= 60) {
            
            timeStamp = [[NSString alloc] initWithFormat:@"%.1f s",interval];
        }else if ((interval>60) && (interval<=60*60)) {
            
            timeStamp = [[NSString alloc] initWithFormat:@"%.1f min",interval/60];
        }else if ((interval>60*60)&&(interval<=60*60*24)) {
            
            timeStamp = [[NSString alloc] initWithFormat:@"%.1f hour",interval/60/60];
        }else {
            
            timeStamp = [[NSString alloc] initWithFormat:@"%.1f day",interval/60/60/24];
        }
        
        return timeStamp;
    }
    return @"";
}

#pragma mark - Pull Refresh 下拉刷新

/**
 *  集成刷新控件
 */
- (void)setupRefresh
{
    [self.tableView addHeaderWithTarget:self action:@selector(scanBLEPrinter) dateKey:@"table"];
    [self.tableView headerBeginRefreshing];
    self.tableView.headerPullToRefreshText = @"Pull down to re-scan";
    self.tableView.headerReleaseToRefreshText = @"Release to start re-scan";
    self.tableView.headerRefreshingText = @"Scaning...";
}

- (void)connectSwitcher:(UISegmentedControl *)sender {
    
    if (sender.selectedSegmentIndex == 2) {
        
        [self.tableView addHeaderWithTarget:self action:@selector(scanWiFiPrinter) dateKey:@"table"];
        [self.tableView headerBeginRefreshing];
    } else if (sender.selectedSegmentIndex == 1) {
        [self.tableView addHeaderWithTarget:self action:@selector(scanBLEPrinter) dateKey:@"table"];
        [self.tableView headerBeginRefreshing];
    }
    [_tableView reloadData];
}

- (void)scanBLEPrinter
{
    [[PTDispatcher share] scanBluetooth];
    [[PTDispatcher share] whenFindBluetoothAll:^(NSDictionary *printerDict) {
        self.blePrinters = printerDict.allValues;
        [self sortPrinters];
        [self.tableView reloadData];
    }];
    
    [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(stopScanAction:) userInfo:nil repeats:true];
}

- (void)stopScanAction:(NSTimer *)timer
{
    [timer invalidate];
    timer = nil;
    [self.tableView headerEndRefreshing];       //停止刷新动画
}

#pragma mark - WiFi Detect

- (void)initWiFiDataBase {
    
    // 载入 OUI 查询字典，以查询网卡制造商
    if (!self.ouiBaseDict) {
        self.ouiBaseDict = [[NSMutableDictionary alloc] init];
        NSString *path = [[NSBundle mainBundle] pathForResource:@"oui" ofType:@"plist"];
        dispatch_async(dispatch_get_main_queue(), ^{
            
            self.ouiBaseDict = [[NSMutableDictionary alloc] initWithContentsOfFile:path];
            [self.tableView reloadData];
        });
    }
}

#pragma -mark wifi scan delegate

- (void)scanWiFiPrinter {
    
    [self initWiFiDataBase];
    
    self.router = [[PTRouter alloc] init];
    
    if (self.router.connected) {
        [[PTDispatcher share] scanWiFi:^(NSDictionary *printerDict) {
            [self.tableView headerEndRefreshing];       //停止刷新动画
            
            self.wifiPrinters = printerDict.allValues;
            [self.tableView reloadData];
        }];
    } else {
        [[[UIAlertView alloc] initWithTitle:@"" message:NSLocalizedString(@"Please Active WiFi",nil) delegate:nil cancelButtonTitle:NSLocalizedString(@"OK",nil) otherButtonTitles:nil, nil] show];
    }
}

@end
