//
//  PTPrinterPort.m
//  WirelessPrinter
//
//  Created by midmirror on 16/8/25.
//  Copyright © 2016年 midmirror. All rights reserved.
//

#import "PTPrinterPort.h"
#import "ProgressHUD.h"
#import <PrinterSDK/PrinterSDK.h>

@implementation PTPrinterPort

+ (void)sendData:(NSData *)data
{
    [ProgressHUD show:@"Sending..."];
    
    [[PTDispatcher share] sendData:data];
    
    [[PTDispatcher share] whenSendSuccess:^{
        [ProgressHUD showSuccess:@"Send Success!"];
    }];
    
    [[PTDispatcher share] whenSendFailure:^{
        [ProgressHUD showError:@"Unconnect!"];
    }];
}

+ (void)saveBluetoothWithUUID:(NSString *)uuid name:(NSString *)name {
        
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSDictionary *histroyDevice = [userDefaults objectForKey:@"deviceRecord"];
    
    NSMutableDictionary *deviceRecordDict = [[NSMutableDictionary alloc] initWithDictionary:histroyDevice];
    
    NSMutableDictionary *bluetoothDict = [[NSMutableDictionary alloc] init];
    [bluetoothDict setObject:name forKey:@"name"];
    [bluetoothDict setObject:[self getCurrentTimeIntervalSince1970] forKey:@"time"];
    
    [deviceRecordDict setObject:bluetoothDict forKey:uuid];
    
    [userDefaults setObject:deviceRecordDict forKey:@"deviceRecord"];
    [userDefaults synchronize];
}

+ (void)removeBluetoothWithUUID:(NSString *)uuid {
    
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSMutableDictionary *histroyDevice = [[NSMutableDictionary alloc] initWithDictionary:[userDefaults objectForKey:@"deviceRecord"]];
    [histroyDevice removeObjectForKey:uuid];
    
    [userDefaults setObject:histroyDevice forKey:@"deviceRecord"];
    [userDefaults synchronize];
}

/**
 *  获取自1970年到此刻的时间间隔
 *
 *  @return 时间间隔（单位：秒）
 */
+ (NSString *)getCurrentTimeIntervalSince1970
{
    NSDate *nowDate = [NSDate date];
    
    NSTimeInterval time = [nowDate timeIntervalSince1970];
    NSString *timeIntervalSince1970 = [[NSString alloc] initWithFormat:@"%f",time];
    
    return timeIntervalSince1970;
}

+ (NSString *)getNameWithPeripheral:(CBPeripheral *)peripheral andAdvDict:(NSDictionary *)advDict {
    
    /** 优先使用广播中的名字 kCBAdvDataLocalName ，其次才使用蓝牙名字 peripheral.name */
    NSString *bluetoothName = [[NSString alloc] initWithFormat:@"%@",advDict[@"kCBAdvDataLocalName"]];;
    if (!bluetoothName || [bluetoothName isEqualToString:@"(null)"]) { //  不存在、或者为空 @"(null)"
        if (peripheral.name) {
            bluetoothName = peripheral.name;
        } else {
            bluetoothName = @"Unknown";
        }
    }
    
    return bluetoothName;
}

+ (NSString *)getMACAddressWithAdvDict:(NSDictionary *)advDict {
    
    NSData *manufacturerData = advDict[@"kCBAdvDataManufacturerData"];
    NSString *macAddress = [[NSString alloc] initWithData:manufacturerData encoding:NSASCIIStringEncoding];
    
    if (manufacturerData.length == 6) {
        macAddress = [self macAddressFromData:manufacturerData];
    } else {
        macAddress = @"(null)";
    }
    
    return macAddress;
}

+ (NSString *)macAddressFromData:(NSData*)data{
    
    NSString *hexString = [[[[NSString stringWithFormat:@"%@",data]
                             stringByReplacingOccurrencesOfString: @"<" withString: @""]
                            stringByReplacingOccurrencesOfString: @">" withString: @""]
                           stringByReplacingOccurrencesOfString: @" " withString: @""];
    
    NSMutableString *macAddr = [[NSMutableString alloc] initWithFormat:@"MAC: "];
    for (NSInteger i = 0; i < hexString.length; i++) {
        
        char c = [hexString characterAtIndex:i];
        [macAddr appendFormat:@"%c",c];
        if ((i%2 == 1) && (i != hexString.length-1)) {
            [macAddr appendString:@":"];
        }
    }
    
    return macAddr;
}

@end
